@@ -129,13 +129,25 @@ static void MaybeWriteBitFileBuffer(BitFile *bitFilePtr)
    WriteBitFileBuffer(bitFilePtr);
 }
 
+/*  The comment does not need to include a newline. */
+void BitFileMoreComment(BitFile *bitFilePtr,char const *comment)
+{  
+   if(bitFilePtr->traceFILEPtr == NULL || comment == NULL)  /* if the trace file is not open */
+      return;
+   if(fputs(comment,bitFilePtr->traceFILEPtr) == EOF) {
+      fprintf(stderr,"ERROR: Error while writing BitFile trace file.\n");
+      exit(-1);
+   };
+   fflush(bitFilePtr->traceFILEPtr);
+}
+
 /*  The comment does not need to include a newline. */
 /*  A null comment is different than a comment of zero length. */
 static void WriteComment(BitFile *bitFilePtr,unsigned int bits,int bitCount,char const *comment)
 {  
    if(bitFilePtr->traceFILEPtr == NULL || comment == NULL)  /* if the trace file is not open */
       return;
-   if(fprintf(bitFilePtr->traceFILEPtr,"%6d.%d %2d %8x %s\n",bitFilePtr->totalBitsProcessed / 8,
+   if(fprintf(bitFilePtr->traceFILEPtr,"\n%6d.%d %2d %8x %s",bitFilePtr->totalBitsProcessed / 8,
               bitFilePtr->totalBitsProcessed % 8,bitCount,bits,comment) < 0) {
       fprintf(stderr,"ERROR: Error while writing BitFile trace file.\n");
       exit(-1);
