@@ -29,6 +29,7 @@ int foundGarbageFlag;
 int numPesPacketsFound;
 int numPacksFound;
 #endif
+int numIFrames,numPFrames,numBFrames;
 
 int progressive_sequence;
 #define CHROMA_FORMAT_420 (1)
@@ -44,6 +45,7 @@ int low_delay;
 int quantiser_scale_code;   
 
 int picture_coding_type;
+int forward_f_code,backward_f_code;
 int fcode00,fcode01,fcode10,fcode11;
 int intra_dc_precision;
 #define TOP_FIELD (1)
@@ -67,43 +69,31 @@ int scalable_mode;
 static void ParseSequenceHeader(BitFile *bitFilePtr)
 {
    int load_intra_quantiser_matrix,load_non_intra_quantiser_matrix,aspect_ratio_information,frame_rate_code;
-   static char const *aspectRatioNames[] = {"aspect_ratio_information = forbidden",
-                                           "aspect_ratio_information = square",
-                                           "aspect_ratio_information = 3 by 4",
-                                           "aspect_ratio_information = 9 by 16",
-                                           "aspect_ratio_information = 1 by 2.21",
-                                           "aspect_ratio_information = reserved"};
-   static char const *frameRateNames[] = {"frame_rate_code = forbidden",
-                                         "frame_rate_code = 24000/1001",
-                                         "frame_rate_code = 24",
-                                         "frame_rate_code = 25",
-                                         "frame_rate_code = 30000/1001",
-                                         "frame_rate_code = 30",
-                                         "frame_rate_code = 50",
-                                         "frame_rate_code = 60000/1001",
-                                         "frame_rate_code = 60",
-                                         "frame_rate_code = reserved"};
-   static char horizSizeComment[50] = "horizontal_size_value = ";
-   static char vertSizeComment[50] =  "vertical_size_value = ";
+   static char const *aspectRatioNames[] = {"forbidden","square","3 by 4","9 by 16","1 by 2.21","reserved"};
+   static char const *frameRateNames[] = {"forbidden","24000/1001","24","25",
+                           "30000/1001","30","50","60000/1001","60","reserved"};
+   char sizeComment[50];
 
    BitFileReadBits(bitFilePtr,32,"sequence_header_code");
-   horizontal_size = BitFilePeekBits(bitFilePtr,12,NULL);
-   sprintf(horizSizeComment+24,"%d",horizontal_size);
-   BitFileReadBits(bitFilePtr,12,horizSizeComment);
-   vertical_size = BitFilePeekBits(bitFilePtr,12,NULL);
-   sprintf(vertSizeComment+22,"%d",vertical_size);
-   BitFileReadBits(bitFilePtr,12,vertSizeComment);
-   aspect_ratio_information = BitFilePeekBits(bitFilePtr,4,NULL);
+   horizontal_size = BitFileReadBits(bitFilePtr,12,"horizontal_size_value = ");
+   sprintf(sizeComment,"%d",horizontal_size);
+   BitFileMoreComment(bitFilePtr,sizeComment);
+   vertical_size = BitFileReadBits(bitFilePtr,12,"vertical_size_value = ");
+   sprintf(sizeComment,"%d",vertical_size);
+   BitFileMoreComment(bitFilePtr,sizeComment);
+   aspect_ratio_information = BitFileReadBits(bitFilePtr,4,"aspect_ratio_information = ");
    if(aspect_ratio_information > 5) 
       aspect_ratio_information = 5;
-   BitFileReadBits(bitFilePtr,4,aspectRatioNames[aspect_ratio_information]);
-   frame_rate_code = BitFilePeekBits(bitFilePtr,4,NULL);
+   BitFileMoreComment(bitFilePtr,aspectRatioNames[aspect_ratio_information]);
+   frame_rate_code = BitFileReadBits(bitFilePtr,4,"frame_rate_code = ");
    if(frame_rate_code > 9)
       frame_rate_code = 9;
-   BitFileReadBits(bitFilePtr,4,frameRateNames[frame_rate_code]);
+   BitFileMoreComment(bitFilePtr,frameRateNames[frame_rate_code]);
    bit_rate = BitFileReadBits(bitFilePtr,18,"bit_rate_value");
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,5);  /* error */
+   };
    vbv_buffer_size = BitFileReadBits(bitFilePtr,10,"vbv_buffer_size_value");
    BitFileReadBits(bitFilePtr,1,"constrained_parameters_flag");
    load_intra_quantiser_matrix = BitFileReadBits(bitFilePtr,1,"load_intra_quantiser_matrix");
@@ -122,27 +112,38 @@ static void ParseSequenceHeader(BitFile *bitFilePtr)
 
 static void ParseSequenceExtension(BitFile *bitFilePtr)
 {
-   static char const *chromaFormatNames[] = {"chroma_format = reserved",
-                                             "chroma_format = 4:2:0",
-                                             "chroma_format = 4:2:2",
-                                             "chroma_format = 4:4:4"};
+   int profileId,levelId;
+   static char const *chromaFormatNames[] = {"reserved", "4:2:0", "4:2:2", "4:4:4"};
+   static char const *profileIdNames[] = {"reserved", "high", "spatially scalable", "SNR scalable", "main",
+                                          "simple", "reserved", "high"};
+   static char const *levelIdNames[] = {"high", "reserved", "high 1440", "reserved", "main", "reserved", "low"};
 
    BitFileReadBits(bitFilePtr,4,"extension_start_code_identifier = sequence extension");
-   BitFileReadBits(bitFilePtr,8,"profile_and_level_indication");
+   BitFileReadBits(bitFilePtr,1,"profile_and_level_indication escape bit");
+   profileId = BitFileReadBits(bitFilePtr,3,"profile id = ");
+   if(profileId > 5)
+      profileId = 0;
+   BitFileMoreComment(bitFilePtr,profileIdNames[profileId]);
+   levelId = BitFileReadBits(bitFilePtr,4,"level id = ");
+   if(levelId < 4 || levelId > 10)
+      levelId = 5;
+   BitFileMoreComment(bitFilePtr,levelIdNames[levelId - 4]);
    progressive_sequence = BitFileReadBits(bitFilePtr,1,"progressive_sequence");
-   chroma_format = BitFilePeekBits(bitFilePtr,2,NULL);
+   chroma_format = BitFileReadBits(bitFilePtr,2,"chroma_format = ");
    if(chroma_format == CHROMA_FORMAT_420)
       block_count = 6;
    else if(chroma_format == CHROMA_FORMAT_422)
       block_count = 8;
    else /* if(chroma_format == CHROMA_FORMAT_444) */
       block_count = 12;
-   BitFileReadBits(bitFilePtr,2,chromaFormatNames[chroma_format]);
+   BitFileMoreComment(bitFilePtr,chromaFormatNames[chroma_format]);
    horizontal_size = (horizontal_size&0xfff) | (BitFileReadBits(bitFilePtr,2,"horizontal_size_extension") << 12);
    vertical_size = (vertical_size&0xfff) | (BitFileReadBits(bitFilePtr,2,"vertical_size_extension") << 12);
    bit_rate = (bit_rate&0xfff) | (BitFileReadBits(bitFilePtr,12,"bit_rate_extension") << 12);
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,6);  /* error */
+   };
    vbv_buffer_size = (vbv_buffer_size&0x3ff) | (BitFileReadBits(bitFilePtr,8,"vbv_buffer_size_extension") << 10);
    low_delay = BitFileReadBits(bitFilePtr,1,"low_delay");
    BitFileReadBits(bitFilePtr,2,"frame_rate_extension_n");
@@ -152,18 +153,11 @@ static void ParseSequenceExtension(BitFile *bitFilePtr)
 static void ParseSequenceDisplayExtension(BitFile *bitFilePtr)
 {
    int colour_description,video_format;
-   static char const *videoFormatNames[] = {"video_format = component",
-                                           "video_format = PAL",
-                                           "video_format = NTSC",
-                                           "video_format = SECAM",
-                                           "video_format = MAC",
-                                           "video_format = unspecified",
-                                           "video_format = reserved",
-                                           "video_format = reserved"};
+   static char const *videoFormatNames[] = {"component","PAL","NTSC","SECAM","MAC","unspecified","reserved","reserved"};
 
    BitFileReadBits(bitFilePtr,4,"extension_start_code_identifier = sequence display");
-   video_format = BitFilePeekBits(bitFilePtr,3,NULL);
-   BitFileReadBits(bitFilePtr,3,videoFormatNames[video_format]);
+   video_format = BitFileReadBits(bitFilePtr,3,"video_format = ");
+   BitFileMoreComment(bitFilePtr,videoFormatNames[video_format]);
    colour_description = BitFileReadBits(bitFilePtr,1,"colour_description");
    if(colour_description) {
       BitFileReadBits(bitFilePtr,8,"colour_primaries");
@@ -171,8 +165,10 @@ static void ParseSequenceDisplayExtension(BitFile *bitFilePtr)
       BitFileReadBits(bitFilePtr,8,"matrix_coefficients");
    };
    BitFileReadBits(bitFilePtr,14,"display_horizontal_size");
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,7);  /* error */
+   };
    BitFileReadBits(bitFilePtr,14,"display_vertical_size");
 }
 
@@ -184,8 +180,10 @@ static void ParseSequenceScalableExtension(BitFile *bitFilePtr)
       BitFileReadBits(bitFilePtr,2,"scalable_mode = spatial scalability");
       BitFileReadBits(bitFilePtr,4,"layer_id");
       BitFileReadBits(bitFilePtr,14,"lower_layer_prediction_horizontal_size");
-      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+         BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
          longjmp(resyncJmpBuf,8);  /* error */
+      };
       BitFileReadBits(bitFilePtr,14,"lower_layer_prediction_vertical_size");
       BitFileReadBits(bitFilePtr,5,"horizontal_subsampling_factor_m");
       BitFileReadBits(bitFilePtr,5,"horizontal_subsampling_factor_n");
@@ -218,38 +216,54 @@ static void ParseUserData(BitFile *bitFilePtr)
 
 static void ParseGroupOfPicturesHeader(BitFile *bitFilePtr)
 {
+   int time_code;
+   char timeStr[200];
+
    BitFileReadBits(bitFilePtr,32,"group_start_code");   /* 0x000001?? */
-   BitFileReadBits(bitFilePtr,25,"time_code");
+   time_code = BitFileReadBits(bitFilePtr,25,"time_code");
+   sprintf(timeStr," = %s%02d:%02d:%02d frame %02d",(time_code&0x1000)?"drop-frame ":"",
+           (time_code >> 19)&0x1f, (time_code >> 13)&0x3f, (time_code >> 6)&0x3f, time_code&0x3f);
+   BitFileMoreComment(bitFilePtr,timeStr);
    BitFileReadBits(bitFilePtr,1,"closed_gop");
    BitFileReadBits(bitFilePtr,1,"broken_link");
    BitFileReadBits(bitFilePtr,5,"padding (should be zero)");
 }
 
+static void AddFCodeComment(BitFile *bitFilePtr,int fCode)
+{
+   static char const *fCodeNames[] = {"forbidden", "[-8,7.5]","[-16,15.5]","[-32,31.5]","[-64,63.5]",
+                "[-128,127.5]","[-256,255.5]","[-512,511.5]","[-1024,1023.5]","[-2048,2047.5]", 
+                "reserved","reserved","reserved","reserved","reserved","not used"};
+
+   BitFileMoreComment(bitFilePtr,fCodeNames[fCode]);
+}
+
 static void ParsePictureHeader(BitFile *bitFilePtr)
 {
    int bitsToPad;
-
-   static char *pictureCodingTypeNames[] = {"picture_coding_type = forbidden",
-                                            "picture_coding_type = intra-coded (I)",
-                                            "picture_coding_type = predictive-coded (P)",
-                                            "picture_coding_type = bidirectionally-predictive-coded (B)",
-                                            "picture_coding_type = dc intra-coded (D)",
-                                            "picture_coding_type = reserved",
-                                            "picture_coding_type = reserved",
-                                            "picture_coding_type = reserved"};
+   static char *pictureCodingTypeNames[] = {"forbidden", "intra-coded (I)", "predictive-coded (P)",
+      "bidirectionally-predictive-coded (B)", "dc intra-coded (D)", "reserved", "reserved", "reserved"};
 
    BitFileReadBits(bitFilePtr,32,"picture_start_code");
    BitFileReadBits(bitFilePtr,10,"temporal_reference");
-   picture_coding_type = BitFilePeekBits(bitFilePtr,3,NULL);
-   BitFileReadBits(bitFilePtr,3,pictureCodingTypeNames[picture_coding_type]);
+   picture_coding_type = BitFileReadBits(bitFilePtr,3,"picture_coding_type = ");
+   BitFileMoreComment(bitFilePtr,pictureCodingTypeNames[picture_coding_type]);
+   if(picture_coding_type == 1)
+      numIFrames++;
+   else if(picture_coding_type == 2)
+      numPFrames++;
+   else if(picture_coding_type == 3)
+      numBFrames++;
    BitFileReadBits(bitFilePtr,16,"vbv_delay");
    if(picture_coding_type == 2 || picture_coding_type == 3) {
       BitFileReadBits(bitFilePtr,1,"full_pel_forward_vector");
-      BitFileReadBits(bitFilePtr,3,"forward_f_code");
+      forward_f_code = BitFileReadBits(bitFilePtr,3,"forward_f_code = ");
+      AddFCodeComment(bitFilePtr,forward_f_code);
    };
    if(picture_coding_type == 3) {
       BitFileReadBits(bitFilePtr,1,"full_pel_backward_vector");
-      BitFileReadBits(bitFilePtr,3,"backward_f_code");
+      backward_f_code = BitFileReadBits(bitFilePtr,3,"backward_f_code = ");
+      AddFCodeComment(bitFilePtr,backward_f_code);
    };
    for(;;) {
       if(BitFileReadBits(bitFilePtr,1,"extra_bit_picture") == 0)
@@ -264,24 +278,22 @@ static void ParsePictureHeader(BitFile *bitFilePtr)
 static void ParseCodingExtension(BitFile *bitFilePtr)
 {
    int composite_display_flag,intra_vlc_format;
-   static char const *pictureStructureNames[] = {"picture_structure = reserved",
-                                                 "picture_structure = top field",
-                                                 "picture_structure = bottom field",
-                                                 "picture_structure = frame picture"};
-   static char const *intraDcPrecisionNames[] = {"intra_dc_precision = 8 bits",
-                                                 "intra_dc_precision = 9 bits",
-                                                 "intra_dc_precision = 10 bits",
-                                                 "intra_dc_precision = 11 bits"};
+   static char const *pictureStructureNames[] = {"reserved", "top field", "bottom field", "frame picture"};
+   static char const *intraDcPrecisionNames[] = {"8 bits", "9 bits", "10 bits", "11 bits"};
 
    BitFileReadBits(bitFilePtr,4,"extension_start_code_identifier = picture coding");   
-   fcode00 = BitFileReadBits(bitFilePtr,4,"fcode[0][0]");   
-   fcode01 = BitFileReadBits(bitFilePtr,4,"fcode[0][1]");   
-   fcode10 = BitFileReadBits(bitFilePtr,4,"fcode[1][0]");   
-   fcode11 = BitFileReadBits(bitFilePtr,4,"fcode[1][1]");   
-   intra_dc_precision = BitFilePeekBits(bitFilePtr,2,NULL);   
-   BitFileReadBits(bitFilePtr,2,intraDcPrecisionNames[intra_dc_precision]);   
-   picture_structure = BitFilePeekBits(bitFilePtr,2,NULL);   
-   BitFileReadBits(bitFilePtr,2,pictureStructureNames[picture_structure]);   
+   fcode00 = BitFileReadBits(bitFilePtr,4,"fcode[0][0] = ");   
+   AddFCodeComment(bitFilePtr,fcode00);
+   fcode01 = BitFileReadBits(bitFilePtr,4,"fcode[0][1] = ");   
+   AddFCodeComment(bitFilePtr,fcode01);
+   fcode10 = BitFileReadBits(bitFilePtr,4,"fcode[1][0] = ");   
+   AddFCodeComment(bitFilePtr,fcode10);
+   fcode11 = BitFileReadBits(bitFilePtr,4,"fcode[1][1] = ");   
+   AddFCodeComment(bitFilePtr,fcode11);
+   intra_dc_precision = BitFileReadBits(bitFilePtr,2,"intra_dc_precision = ");   
+   BitFileMoreComment(bitFilePtr,intraDcPrecisionNames[intra_dc_precision]);   
+   picture_structure = BitFileReadBits(bitFilePtr,2,"picture_structure = ");   
+   BitFileMoreComment(bitFilePtr,pictureStructureNames[picture_structure]);   
    BitFileReadBits(bitFilePtr,1,"top_field_first");   
    frame_pred_frame_dct = BitFileReadBits(bitFilePtr,1,"frame_pred_frame_dct");   
    concealment_motion_vectors = BitFileReadBits(bitFilePtr,1,"concealment_motion_vectors");   
@@ -347,11 +359,15 @@ static void ParsePictureDisplayExtension(BitFile *bitFilePtr)
 #if 0
    for(frameCenterOffsetIndex=0;frameCenterOffsetIndex < ;frameCenterOffsetIndex) {
       BitFileReadBits(bitFilePtr,16,"frame_centre_horizontal_offset");   
-      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+         BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
          longjmp(resyncJmpBuf,9);  /* error */
+      };
       BitFileReadBits(bitFilePtr,16,"frame_centre_vertical_offset");   
-      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {    
+         BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
          longjmp(resyncJmpBuf,10);  /* error */
+      };
    };
 #endif
 }
@@ -361,8 +377,10 @@ static void ParsePictureTemporalScalableExtension(BitFile *bitFilePtr)
    BitFileReadBits(bitFilePtr,4,"extension_start_code_identifier = picture temporal scalable");   
    BitFileReadBits(bitFilePtr,2,"referece_select_code");   
    BitFileReadBits(bitFilePtr,10,"forward_temporal_reference");   
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,11);  /* error */
+   };
    BitFileReadBits(bitFilePtr,10,"backward_temporal_reference");   
 }
 
@@ -370,11 +388,15 @@ static void ParsePictureSpatialScalableExtension(BitFile *bitFilePtr)
 {
    BitFileReadBits(bitFilePtr,4,"extension_start_code_identifier = picture spatial scalable");   
    lower_layer_temporal_reference = BitFileReadBits(bitFilePtr,10,"lower_layer_temporal_reference");   
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,12);  /* error */
+   };
    lower_layer_horizontal_offset = BitFileReadBits(bitFilePtr,10,"lower_layer_horizontal_offset");   
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,13);  /* error */
+   };
    lower_layer_vertical_offset = BitFileReadBits(bitFilePtr,10,"lower_layer_vertical_offset");   
    spatial_temporal_weight_code_table_index = BitFileReadBits(bitFilePtr,2,"spatial_temporal_weight_code_table_index");   
    lower_layer_progressive_frame = BitFileReadBits(bitFilePtr,1,"lower_layer_progressive_frame");   
@@ -388,14 +410,20 @@ static void ParseCopyrightExtension(BitFile *bitFilePtr)
    BitFileReadBits(bitFilePtr,8,"copyright_identifier");   
    BitFileReadBits(bitFilePtr,1,"original_or_copy");   
    BitFileReadBits(bitFilePtr,7,"reserved");   
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,14);  /* error */
+   };
    BitFileReadBits(bitFilePtr,20,"copyright_number_1");     /* should be 22? */
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,15);  /* error */
+   };
    BitFileReadBits(bitFilePtr,22,"copyright_number_2");   
-   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+   if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+      BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
       longjmp(resyncJmpBuf,16);  /* error */
+   };
    BitFileReadBits(bitFilePtr,22,"copyright_number_3");   
 }
 
@@ -450,14 +478,14 @@ static void ParseExtension(BitFile *bitFilePtr)
       BitFileReadBits(bitFilePtr,bitsToPad,"padding (should be 0)");
 }
 
-static void ParseMotionVectors(BitFile *bitFilePtr,int s)
+static void ParseMotionVectors(BitFile *bitFilePtr,int motion_vector_count,int dmvPresentFlag)
 {
-
-
    extern HuffmanTable *huffmanMotion;
-  
-   huffmanMotion = NULL;
 
+   for(;motion_vector_count;motion_vector_count--) {
+  
+      huffmanMotion = NULL;
+   };
 }
 
 
@@ -465,60 +493,81 @@ static void ParseDctCoefficients(BitFile *bitFilePtr,int firstCoefIndex,HuffmanT
 {
    short dctArr[64];
    int dctIndex,sign,runLevel,run,level;
+   char commentStr[100];
 
    dctIndex = firstCoefIndex;
    if(firstCoefIndex == 0 && huffmanTablePtr == huffmanDctCoefZero) {
       if(BitFilePeekBits(bitFilePtr,1,NULL) == 1) {
          BitFilePeekBits(bitFilePtr,1,"  DC DCT coef NOTE3");
-         if(BitFilePeekBits(bitFilePtr,1,"sign"))
+         if(BitFilePeekBits(bitFilePtr,1,"sign = ")) {
+            BitFileMoreComment(bitFilePtr,"-");
             dctArr[0] = -1;
-         else
+         } else {
+            BitFileMoreComment(bitFilePtr,"+");
             dctArr[0] = 1;
+         };
          dctIndex = 1;
       };
    };
    for(;;) {
-      runLevel = ReadHuffmanCode(bitFilePtr,huffmanTablePtr,"  DCT Coef");
+      runLevel = ReadHuffmanCode(bitFilePtr,huffmanTablePtr,"  DCT Coef = ");
       if(runLevel == HUFFMAN_ERROR) {
+         BitFileMoreComment(bitFilePtr,"\n*** next bits not valid DCT variable length code");
          longjmp(resyncJmpBuf,17);  /* error */
       } else if(runLevel == END_OF_BLOCK) {
-         if(dctIndex == firstCoefIndex)
-            longjmp(resyncJmpBuf,1);  /* error */
+         BitFileMoreComment(bitFilePtr,"end of block");
          break;
       } else if(runLevel == ESCAPE) {
+         BitFileMoreComment(bitFilePtr,"escape");
          run = BitFileReadBits(bitFilePtr,6,"  run");
          sign = BitFileReadBits(bitFilePtr,1,"  sign");
          level = BitFileReadBits(bitFilePtr,11,"  level");
-         if(level == 0)
+         if(level == 0) {
+            BitFileMoreComment(bitFilePtr,"\n*** level code of 0 was not legal");
             longjmp(resyncJmpBuf,2);  /* error */
+         };
          if(sign)
             dctArr[dctIndex] = -2048 + level;
          else 
             dctArr[dctIndex] = level;
       } else {  /* else not an escape or end-of-block */
          level = runLevel & 0xff;
-         if(BitFileReadBits(bitFilePtr,1,"  sign"))
-            dctArr[dctIndex] = -level;
-         else
-            dctArr[dctIndex] = level;
          run = runLevel >> 8;
+         sprintf(commentStr," run=%d level=%d",run,level);
+         BitFileMoreComment(bitFilePtr,commentStr);
+         if(BitFileReadBits(bitFilePtr,1,"  sign = ")) {
+            BitFileMoreComment(bitFilePtr,"-");
+            dctArr[dctIndex] = -level;
+         } else {
+            BitFileMoreComment(bitFilePtr,"+");
+            dctArr[dctIndex] = level;
+         };
       };
       assert(run >= 0 && run < 64);
       dctIndex += 1 + run;
-      if(dctIndex >= 64)
+      if(dctIndex >= 64) {
+         BitFileMoreComment(bitFilePtr,"\n*** run took us past 64 coefficients");
          longjmp(resyncJmpBuf,3);  /* error */
+      };
    };
-   BitFileReadBits(bitFilePtr,2,"end of block");
 }
 
 int useIntraDcPredictor;
 
 static void ParseMacroblock(BitFile *bitFilePtr)
 {
-   int macroblock_address_increment,macroblock_type,dct_type;
-   int field_motion_type,frame_motion_type,coded_block_pattern,i;
+   int macroblock_address_increment,macroblock_type,dct_type,spatial_temporal_weight_code;
+   int motion_type,coded_block_pattern,i,motion_vector_count;
+   static const char *frameMotionNames[] = {"reserved","field-based","frame-based","dual-prime"};
+   static const char *fieldMotionNames[] = {"reserved","field-based","16x8 MC","dual-prime"};
+   static const int spatialTemporalClasses[] = { 1,1,1,1,3,1,3,1,2,1,2,1,2,2,3,1 };
+   char commentStr[100];
 
-   macroblock_address_increment = ReadHuffmanCode(bitFilePtr,huffmanMbAddressing," macroblock_address_increment");
+   macroblock_address_increment = ReadHuffmanCode(bitFilePtr,huffmanMbAddressing," macroblock_address_increment = ");
+   if(macroblock_address_increment == HUFFMAN_ERROR)
+      longjmp(resyncJmpBuf,18);  /* go resync to a start code */ 
+   sprintf(commentStr,"%d",macroblock_address_increment);
+   BitFileMoreComment(bitFilePtr,commentStr);
    if(macroblock_address_increment)
       useIntraDcPredictor = 0;
    if(picture_coding_type == 1) {  /* if I picture */
@@ -537,14 +586,32 @@ static void ParseMacroblock(BitFile *bitFilePtr)
       else
          macroblock_type = ReadHuffmanCode(bitFilePtr,huffmanMbTypeForB,"  macroblock_type");
    };
+   if(macroblock_type == HUFFMAN_ERROR)
+      longjmp(resyncJmpBuf,19);  /* go resync to a start code */ 
    if((macroblock_type & MB_SPATIAL_TEMPORAL_WEIGHT_CODE_FLAG) && spatial_temporal_weight_code_table_index != 0) 
-       BitFileReadBits(bitFilePtr,2,"spatial_temporal_weight_code");   
+       spatial_temporal_weight_code = BitFileReadBits(bitFilePtr,2,"spatial_temporal_weight_code");   
    if((macroblock_type & (MB_MOTION_FORWARD | MB_MOTION_BACKWARD))) {
       if(picture_structure == FRAME_PICTURE) {
-         if(frame_pred_frame_dct == 0)
-            frame_motion_type = BitFileReadBits(bitFilePtr,2,"frame_motion_type");   
+         if(frame_pred_frame_dct == 0) {
+            motion_type = BitFileReadBits(bitFilePtr,2,"frame_motion_type = ");   
+            BitFileMoreComment(bitFilePtr,frameMotionNames[motion_type]); 
+         };
+         if(motion_type == 1 && (spatialTemporalClasses[spatial_temporal_weight_code_table_index*4 + 
+            spatial_temporal_weight_code] > 2))  /* spatial_temporal_weight_class */
+            motion_vector_count = 2;
+         else 
+            motion_vector_count = 1;
       } else {
-         field_motion_type = BitFileReadBits(bitFilePtr,2,"field_motion_type");   
+         motion_type = BitFileReadBits(bitFilePtr,2,"field_motion_type");   
+         BitFileMoreComment(bitFilePtr,fieldMotionNames[motion_type]); 
+         if(motion_type == 2)
+            motion_vector_count = 2;
+         else 
+            motion_vector_count = 1;
+      };
+      if(motion_type == 0) {
+         BitFileMoreComment(bitFilePtr,"\n*** reserved motion type not permitted");
+         longjmp(resyncJmpBuf,1);  /* error */
       };
    };
    if(picture_structure == FRAME_PICTURE && frame_pred_frame_dct == 0 && (macroblock_type & (MB_INTRA | MB_PATTERN)))
@@ -552,14 +619,18 @@ static void ParseMacroblock(BitFile *bitFilePtr)
    if((macroblock_type & MB_QUANT))
       quantiser_scale_code = BitFileReadBits(bitFilePtr,1,"quantiser_scale_code");   
    if((macroblock_type & MB_MOTION_FORWARD) || ((macroblock_type & MB_INTRA) && concealment_motion_vectors))
-      ParseMotionVectors(bitFilePtr,0);
+      ParseMotionVectors(bitFilePtr,motion_vector_count,motion_type == 3);
    if((macroblock_type & MB_MOTION_BACKWARD))
-      ParseMotionVectors(bitFilePtr,1);
+      ParseMotionVectors(bitFilePtr,motion_vector_count,motion_type == 3);
    if((macroblock_type & MB_INTRA) && concealment_motion_vectors)
-      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0)
+      if(BitFileReadBits(bitFilePtr,1,"marker_bit (must be 1)") == 0) {
+         BitFileMoreComment(bitFilePtr,"\n*** bad marker bit");
          longjmp(resyncJmpBuf,4);  /* error */
+      };
    if((macroblock_type & MB_PATTERN)) {  
       coded_block_pattern = ReadHuffmanCode(bitFilePtr,huffmanCodedBlockPattern,"  coded_block_pattern_420") << 6;
+      if(coded_block_pattern == HUFFMAN_ERROR)
+         longjmp(resyncJmpBuf,20);  /* go resync to a start code */ 
       if(chroma_format == CHROMA_FORMAT_422) 
          coded_block_pattern |= BitFileReadBits(bitFilePtr,2,"coded_block_pattern_1") << 4;   
       if(chroma_format == CHROMA_FORMAT_444)
@@ -571,26 +642,18 @@ static void ParseMacroblock(BitFile *bitFilePtr)
       }
    } else if((macroblock_type & MB_INTRA)) { /* else no coded_block_pattern so if intra */
       int dct_dc_size_luminance,dct_dc_size_chrominance;
-      i = 0;
-      if(!useIntraDcPredictor) {
-         BitFileReadBits(bitFilePtr,8+intra_dc_precision,"intra DC (no prediction)");   
-         ParseDctCoefficients(bitFilePtr,1,intraDctHuffmanTablePtr);
-         i++;
-      };
-      for(;i < 4;i++) {
+      for(i=0;i < 4;i++) {  /* for the luminance blocks */
          dct_dc_size_luminance = ReadHuffmanCode(bitFilePtr,huffmanDctDcSizeLum,"  dct_dc_size_luminance");
+         if(dct_dc_size_luminance == HUFFMAN_ERROR)
+            longjmp(resyncJmpBuf,21);  /* go resync to a start code */ 
          if(dct_dc_size_luminance)
             BitFileReadBits(bitFilePtr,dct_dc_size_luminance,"   dct_dc_differential");   
          ParseDctCoefficients(bitFilePtr,0,intraDctHuffmanTablePtr);
       };
-      if(!useIntraDcPredictor) {
-         BitFileReadBits(bitFilePtr,8+intra_dc_precision,"intra DC (no prediction)");   
-         ParseDctCoefficients(bitFilePtr,1,intraDctHuffmanTablePtr);
-         BitFileReadBits(bitFilePtr,8+intra_dc_precision,"intra DC (no prediction)");   
-         ParseDctCoefficients(bitFilePtr,1,intraDctHuffmanTablePtr);
-      };
-      for(;i < block_count;i++) {
-         dct_dc_size_chrominance = ReadHuffmanCode(bitFilePtr,huffmanDctDcSizeLum,"  dct_dc_size_chrominance");
+      for(;i < block_count;i++) {  /* for the chrominance blocks */
+         dct_dc_size_chrominance = ReadHuffmanCode(bitFilePtr,huffmanDctDcSizeChrom,"  dct_dc_size_chrominance");
+         if(dct_dc_size_chrominance == HUFFMAN_ERROR)
+            longjmp(resyncJmpBuf,22);  /* go resync to a start code */ 
          if(dct_dc_size_chrominance)
             BitFileReadBits(bitFilePtr,dct_dc_size_chrominance,"   dct_dc_differential");   
          ParseDctCoefficients(bitFilePtr,0,intraDctHuffmanTablePtr);
@@ -613,12 +676,13 @@ static void ParsePictureData(BitFile *bitFilePtr)
       BitFileReadBits(bitFilePtr,7,"priority_breakpoint");   
    };
 #endif
-   quantiser_scale_code = BitFileReadBits(bitFilePtr,7,"quantiser_scale_code");   
+   quantiser_scale_code = BitFileReadBits(bitFilePtr,5,"quantiser_scale_code");   
    extra_bit_slice  = BitFilePeekBits(bitFilePtr,1,NULL);
    if(extra_bit_slice) {  
       BitFileReadBits(bitFilePtr,1,"intra_slice_flag"); /* would be 1 */
       BitFileReadBits(bitFilePtr,1,"intra_slice");   
-      BitFileReadBits(bitFilePtr,7,"reserved_bits");   
+      BitFileReadBits(bitFilePtr,1,"slice_picture_id_enable");   
+      BitFileReadBits(bitFilePtr,6,"slice_picture_id");   
       for(;;) {
          extra_bit_slice = BitFilePeekBits(bitFilePtr,1,NULL);
          if(extra_bit_slice == 0)    
@@ -627,7 +691,7 @@ static void ParsePictureData(BitFile *bitFilePtr)
          BitFileReadBits(bitFilePtr,8,"extra_infomation_slice");   
       };
    };
-   BitFilePeekBits(bitFilePtr,1,"extra_bit_slice");  /* would be 0 */
+   BitFileReadBits(bitFilePtr,1,"extra_bit_slice");  /* would be 0 */
    useIntraDcPredictor = 0;
    while(BitFilePeekBits(bitFilePtr,23,NULL) != 0) {
       ParseMacroblock(bitFilePtr);
@@ -636,9 +700,11 @@ static void ParsePictureData(BitFile *bitFilePtr)
 
 static void ParseVideo(BitFile *bitFilePtr)
 {
-   int bitsToSkip;
+   int bitsToSkip,errorCode;
 
-   setjmp(resyncJmpBuf);
+   if((errorCode = setjmp(resyncJmpBuf))) {
+      fprintf(stderr,"error %d bitFileFTell=%d.%d\n",errorCode,BitFileFTell(bitFilePtr)/8,BitFileFTell(bitFilePtr)%8);
+   };
    for(;;) {  /* for every byte until we find a start code. */
       if(BitFilePeekBits(bitFilePtr,24,NULL) == 0x000001) { /* if a start code */
          if(BitFilePeekBits(bitFilePtr,32,NULL) == 0x00000100) { /* if a picture_start_code */
@@ -676,7 +742,7 @@ static void PrintHelp(void)
    printf("   --input=filename (default is vts_01_1.mp2)\n"); 
    printf("   --bittrace=filename (default is videotrace)\n"); 
    printf("\n"); 
-   printf("  videobits version 1.1\n"); 
+   printf("  videobits version 1.2\n"); 
    printf("  Written by Rick Dean <vobTools@fdd.com>.\n");
    printf("  Copyright (c) 1999 Rick Dean."); 
    printf("  Released under version 2 of the GNU Public License.\n");
